#include "Integrator.h"
#include "Quaternion.h"
#include "Arduino.h"

Integrator::Integrator(float initial /* = 0 */)
    : value_(initial), speed_(0), prev_(0)
{
}

float Integrator::value() const { return value_; }
float Integrator::speed() const { return speed_; }

float Integrator::accumulate(float accel, float dt)
{
    float v = speed_ + accel * dt / 2;
    float dx = v * dt;
    value_ += dx;
    speed_ += accel * dt;
    return dx;
}

INS::INS()
    : gx_(0), gy_(0), gz_(0)
{
}

float INS::posX() const { return posX_.value(); }
float INS::posY() const { return posY_.value(); }
float INS::posZ() const { return posZ_.value(); }
float INS::rotX() const { return rotX_.value(); }
float INS::rotY() const { return rotY_.value(); }
float INS::rotZ() const { return rotZ_.value(); }

float INS::posDX() const { return posX_.speed(); }
float INS::posDY() const { return posY_.speed(); }
float INS::posDZ() const { return posZ_.speed(); }
float INS::rotDX() const { return rotX_.speed(); }
float INS::rotDY() const { return rotY_.speed(); }
float INS::rotDZ() const { return rotZ_.speed(); }

float INS::grvX() const { return gx_; }
float INS::grvY() const { return gy_; }
float INS::grvZ() const { return gz_; }

void INS::setGravity(float ax, float ay, float az)
{
    gx_ = ax;
    gy_ = ay;
    gz_ = az;
}

void INS::accumulate(float px, float py, float pz,
    float rx, float ry, float rz, float dt)
{
    float rdx = rotX_.accumulate(rx, dt);
    float rdy = rotY_.accumulate(ry, dt);
    float rdz = rotZ_.accumulate(rz, dt);

    Quaternion q = Quaternion::from_euler_rotation(rdx, rdy, rdz);
    Quaternion g(gx_, gy_, gz_);
    g = q.rotate(g);
    gx_ = g.b;
    gy_ = g.c;
    gz_ = g.d;

    posX_.accumulate(px - gx_, dt);
    posY_.accumulate(py - gy_, dt);
    posZ_.accumulate(pz - gz_, dt);
}
